{ pkgs, lib, writeText, fetchedGitPrivate, stdenv, composerEnv, phpPackages, fetchurl, fetchgit }:
let
  connexionswing = { config }: rec {
    environment = config.environment;
    varDir = "/var/lib/connexionswing_${environment}";
    configRoot =
      writeText "parameters.yml" ''
        # This file is auto-generated during the composer install
        parameters:
            database_host: ${config.mysql.host}
            database_port: ${config.mysql.port}
            database_name: ${config.mysql.name}
            database_user: ${config.mysql.user}
            database_password: ${config.mysql.password}
            database_server_version: ${pkgs.mariadb.mysqlVersion}
            mailer_transport: smtp
            mailer_host: mail.immae.eu
            mailer_user: null
            mailer_password: null
            subscription_email: ${config.email}
            allow_robots: true
            secret: ${config.secret}
      '';
    phpFpm = rec {
      socket = "/var/run/phpfpm/connexionswing-${environment}.sock";
      phpConfig = ''
        extension=${phpPackages.imagick}/lib/php/extensions/imagick.so
        '';
      pool = ''
        listen = ${socket}
        user = ${apache.user}
        group = ${apache.group}
        listen.owner = ${apache.user}
        listen.group = ${apache.group}
        php_admin_value[upload_max_filesize] = 20M
        php_admin_value[post_max_size] = 20M
        ;php_admin_flag[log_errors] = on
        php_admin_value[open_basedir] = "${configRoot}:${webappDir}:${varDir}:/tmp"
        php_admin_value[session.save_path] = "${varDir}/phpSessions"
        ${if environment == "dev" then ''
        pm = ondemand
        pm.max_children = 5
        pm.process_idle_timeout = 60
        env[SYMFONY_DEBUG_MODE] = "yes"
        '' else ''
        pm = dynamic
        pm.max_children = 20
        pm.start_servers = 2
        pm.min_spare_servers = 1
        pm.max_spare_servers = 3
        ''}'';
    };
    apache = {
      user = "wwwrun";
      group = "wwwrun";
      modules = [ "proxy_fcgi" ];
      vhostConf = ''
      <FilesMatch "\.php$">
        SetHandler "proxy:unix:${phpFpm.socket}|fcgi://localhost"
      </FilesMatch>

      <Directory ${varDir}/medias>
        Options FollowSymLinks
        AllowOverride None
        Require all granted
      </Directory>

      <Directory ${varDir}/uploads>
        Options FollowSymLinks
        AllowOverride None
        Require all granted
      </Directory>

      ${if environment == "dev" then ''
      <Location />
        Use LDAPConnect
        Require ldap-group   cn=connexionswing.immae.eu,cn=httpd,ou=services,dc=immae,dc=eu
        ErrorDocument 401 "<html><meta http-equiv=\"refresh\" content=\"0;url=https://connexionswing.com\"></html>"
      </Location>

      <Directory ${webRoot}>
        Options Indexes FollowSymLinks MultiViews Includes
        AllowOverride None
        Require all granted

        DirectoryIndex app_dev.php

        <IfModule mod_negotiation.c>
        Options -MultiViews
        </IfModule>

        <IfModule mod_rewrite.c>
          RewriteEngine On

          RewriteCond %{REQUEST_URI}::$1 ^(/.+)/(.*)::\2$
          RewriteRule ^(.*) - [E=BASE:%1]

          # Maintenance script
          RewriteCond %{DOCUMENT_ROOT}/maintenance.php -f
          RewriteCond %{SCRIPT_FILENAME} !maintenance.php
          RewriteRule ^.*$ %{ENV:BASE}/maintenance.php [R=503,L]
          ErrorDocument 503 /maintenance.php

          # Sets the HTTP_AUTHORIZATION header removed by Apache
          RewriteCond %{HTTP:Authorization} .
          RewriteRule ^ - [E=HTTP_AUTHORIZATION:%{HTTP:Authorization}]

          RewriteCond %{ENV:REDIRECT_STATUS} ^$
          RewriteRule ^app_dev\.php(?:/(.*)|$) %{ENV:BASE}/$1 [R=301,L]

          # If the requested filename exists, simply serve it.
          # We only want to let Apache serve files and not directories.
          RewriteCond %{REQUEST_FILENAME} -f
          RewriteRule ^ - [L]

          # Rewrite all other queries to the front controller.
          RewriteRule ^ %{ENV:BASE}/app_dev.php [L]
        </IfModule>

      </Directory>
      '' else ''
      Use Stats connexionswing.com

      <Directory ${webRoot}>
        Options Indexes FollowSymLinks MultiViews Includes
        AllowOverride All
        Require all granted
      </Directory>
      ''}
      '';
    };
    activationScript = {
      deps = [ "wrappers" ];
      text = ''
      install -m 0755 -o ${apache.user} -g ${apache.group} -d ${varDir} \
        ${varDir}/medias \
        ${varDir}/uploads \
        ${varDir}/var
      install -m 0750 -o ${apache.user} -g ${apache.group} -d ${varDir}/phpSessions
      if [ ! -f "${varDir}/currentWebappDir" -o \
          "${webappDir}" != "$(cat ${varDir}/currentWebappDir 2>/dev/null)" ]; then
        pushd ${webappDir} > /dev/null
        $wrapperDir/sudo -u wwwrun ./bin/console --env=${environment} cache:clear --no-warmup
        popd > /dev/null
        echo -n "${webappDir}" > ${varDir}/currentWebappDir
      fi
      '';
    };
    webappDir = composerEnv.buildPackage (
      import ./php-packages.nix { inherit composerEnv fetchurl fetchgit; } //
      fetchedGitPrivate ./connexionswing.json //
      rec {
        noDev = (environment == "prod");
        preInstall = ''
          export SYMFONY_ENV="${environment}"
          '';
        postInstall = ''
          cd $out
          ${if environment == "prod" then "php ./bin/console assetic:dump --env=prod --no-debug" else ""}
          rm app/config/parameters.yml
          ln -sf ${configRoot} app/config/parameters.yml
          rm -rf var/{logs,cache}
          ln -sf ../../../../../../${varDir}/var/{logs,cache} var/
          ln -sf ../../../../../${varDir}/{medias,uploads} web/images/
          '';
      });
    webRoot = "${webappDir}/web";
  };
in
  connexionswing
